
package shai.vmutils;

import sun.tools.java.Constants;
import java.io.*;

/**
 * each instance of this class represents an entry in the constant pool.
**/
public class ConstantPoolEntry implements sun.tools.java.Constants 
{
	// the type of the entry in the constant pool
	private byte type;
	
	// the object at the entry in the constant pool.
	private Object data;


	/**
	 * construct an entry in the constant pool.
	**/	
	public ConstantPoolEntry(Object data, byte type)
	{
		this.data = data;
		this.type = type;
	}	
/**
 * This factory method creates a contant pool entry. 
 * It was built for use while iterating through the class file header.
 * @return shai.vmutils.ConstantPoolEntry
 * @param type byte
 * @param stream DataInputStream
 */
public static ConstantPoolEntry getConstantPoolEntry ( byte type, DataInputStream stream) 
		throws ClassFormatError, IOException
{
	switch(	type)
	{
		case CONSTANT_UTF8:
			return(new ConstantPoolEntry(stream.readUTF(), type));
		case CONSTANT_INTEGER:
			return(new ConstantPoolEntry(new Integer(stream.readInt()), type));
		case CONSTANT_FLOAT:
			return(new ConstantPoolEntry(new Float(stream.readFloat()), type));
		case CONSTANT_LONG:
			return(new ConstantPoolEntry(new Long(stream.readLong()), type));
		case CONSTANT_DOUBLE:
			return(new ConstantPoolEntry(new Double(stream.readDouble()), type));

		case CONSTANT_CLASS:
		case CONSTANT_STRING:
			return(new ConstantPoolEntry(new Integer(stream.readShort()), type));
				
		case CONSTANT_FIELD:
		case CONSTANT_METHOD:
		case CONSTANT_INTERFACEMETHOD:
		case CONSTANT_NAMEANDTYPE:
			return(new ConstantPoolEntry(new Integer((stream.readShort() << 16) | stream.readShort()), type));

		default:
			throw (new ClassFormatError("Constant type not found: " + type));
	}	
}
/**
 * This method returns this entry as an integer.
 * @return int
 */
public int getInt() {
	return(((Integer)data).intValue());
}

/**
 * This method returns this entry as a String.
 * @return String
 */
public String getString() {
	return((String)data);
}
/**
 * This method returns the type of this entry.
 * @return byte
 */
public byte getType() 
{
	return (type);
}
/**
 * This method returns the type of the current constant pool entry as a String 
 * which is helpfull for displaying the entry information.
 * @return java.lang.String
 */
public String getTypeAsString() 
{
	switch(	type)
	{
		case CONSTANT_UTF8:
			return("UTF8 ");
		case CONSTANT_FLOAT:
			return("Float ");
		case CONSTANT_INTEGER:
			return("Integer ");
		case CONSTANT_LONG:
			return("Long ");
		case CONSTANT_DOUBLE:
			return("Double ");
		case CONSTANT_CLASS:
			return("Class ");
		case CONSTANT_STRING:
			return("String ");
		case CONSTANT_FIELD:
			return("Field ");
		case CONSTANT_METHOD:
			return("Method ");
		case CONSTANT_INTERFACEMETHOD:
			return("Interface method ");
		case CONSTANT_NAMEANDTYPE:
			return("Name and type ");
		default:
			return("Invalid ");
	}	
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isClass() 
{
	return (CONSTANT_CLASS == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isDouble() 
{
	return (CONSTANT_DOUBLE == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isField() 
{
	return (CONSTANT_FIELD == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isFloat() 
{
	return (CONSTANT_FLOAT == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isInteger() 
{
	return (CONSTANT_INTEGER == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isInterfaceMethod() 
{
	return (CONSTANT_INTERFACEMETHOD == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isLong() 
{
	return (CONSTANT_LONG == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isMethod() 
{
	return (CONSTANT_METHOD == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isNameAndType() 
{
	return (CONSTANT_NAMEANDTYPE == type);
}
/**
 * This method returns true if the type of this entry is a class.
 * @return boolean
 */
public boolean isString() 
{
	return (CONSTANT_STRING == type);
}
/**
 * This method returns true when the entry is of UTF8 type.
 * @return boolean
 */
public boolean isUTF8() {
	return (CONSTANT_UTF8 == type);
}
/**
 * This method was created by a SmartGuide.
 * @param outputStream java.io.DataOutputStream
 * @exception java.io.IOException The exception description.
 */
public void save(DataOutputStream outputStream) 
	throws IOException 
{
	// write the type of the entry.
	outputStream.writeByte(type);

	// check the type of the class and write it to the file.		
	switch(	type)
	{
		case CONSTANT_UTF8:
			outputStream.writeUTF((String)data);
			return;
		case CONSTANT_INTEGER:
			outputStream.writeInt(((Integer)data).intValue());
			return;
		case CONSTANT_FLOAT:
			outputStream.writeFloat(((Float)data).floatValue());
			return;
		case CONSTANT_LONG:
			outputStream.writeLong(((Long)data).longValue());
			return;
		case CONSTANT_DOUBLE:
			outputStream.writeDouble(((Double)data).doubleValue());
			return;

		case CONSTANT_CLASS:
		case CONSTANT_STRING:
			outputStream.writeShort((short)((Integer)data).intValue());
			return;
				
		case CONSTANT_FIELD:
		case CONSTANT_METHOD:
		case CONSTANT_INTERFACEMETHOD:
		case CONSTANT_NAMEANDTYPE:
			outputStream.writeShort((short)((Integer)data).intValue() >> 16);
			outputStream.writeShort((short)((Integer)data).intValue() & 0xFFFF);
			return;

		default:
			throw (new ClassFormatError("Constant type not found: " + type));
	}	
}
/**
 * This method returns the value of the current field as a string. It is quite usless without
 * the type which is expressed in the getType as string method.
 * @return java.lang.String
 */
public String toString() 
{
	if(	type == CONSTANT_UTF8)
		return((String)data);
	return(data.toString());
}
}