
package shai.vmutils;

import sun.tools.util.*;
import sun.tools.java.*;
import java.io.*;
import shai.vmutils.ConstantPoolEntry;
import java.util.Vector;

/**
 * the VMAnalizer class analizes the structure of a class file.
 * the Constants interface contains all the predefined constants for
 * class file parsing.
 *
 * This class was developed by studying java's source code.
 * it does not contain any copyrighted material of Sun but it might bare some
 * external simlarities to Sun's classes.
 * Sun provides an excelent class for loding class files called BinaryClass however
 * that class was not designed for the same purposes as this class, however you may
 * find it very useful.
**/
public class VMAnalizer implements Constants
{
	// minor version number 
	private int minorVersion;

	// major version number 
	private int majorVersion;

	// the list of objects which comprises the constant pool.
	private ConstantPoolEntry constantPool[];
	
	// the class modifiers.
	private int modifiers;	

	// the offset in the constant pool of the name of this class and its super class.
	private int className;
	private int superClassName;

	// pointers to the names of all the supported interfaces in the constantPool.
	private int supportedInterfaces[];

	// the array containing the list of all the fields in the class.
	private FieldEntry listOfFields[];

	// the array containing the list of all the methods in the class.
	private MethodEntry listOfMethods[];

	// the attribute table for this class file.
	private AttributeEntry attributeTable;

/**
 * the constructor takes a classFileName which loads the binaryClassRepresentation
 * and enables the interogation of the classes properties.
**/
public VMAnalizer(String classFileName) throws IOException
{
	loadClass(classFileName);
}	
/**
 * This method returns the attributes of the class.
 * @return shai.vmutils.AttributeEntry
 */
public AttributeEntry getClassAttributes() 
{
	return (attributeTable);
}
/**
 * This method returns the name of this class.
 * @return java.lang.String
 */
public String getClassName() 
{
	// the offset in the constant pool of the name of this class.
	return(constantPool[constantPool[className].getInt()]).getString();
}
/**
 * This method returns the number of iterms in the constant pool.
 * @return int
 */
public int getConstantPoolCount() 
{
	return (constantPool.length);
}
/**
 * This method returns the entry at offset.
 * @return java.lang.Object
 * @param offset int
 */
public Object getEntry(int offset) 
{
	return (constantPool[offset]);
}
/**
 * This method returns the type of the entry at offset.
 * @return int
 */
public byte getEntryOffsetType(int offset) 
{
	return (constantPool[offset].getType());
}
/**
 * This method returns the field entry at offset.
 */
public FieldEntry getField(int offset) 
{
	// the array containing the list of all the fields in the class.
	return(listOfFields[offset]);
}
/**
 * This method returns the number of the fields in the class.
 * @return int
 */
public int getFieldCount() 
{
	return (listOfFields.length);
}
/**
 * This method returns the name of the interface on offset interfaceNumber or null if 
 * the number of interfaces is exceeded.
 * @return java.lang.String
 * @param interfaceNumber int
 */
public String getInterfaceAsString(int interfaceNumber) 
{
	if(supportedInterfaces.length > interfaceNumber)
	{
		return(constantPool[supportedInterfaces[interfaceNumber] - 1].getString());
	}	
	return null;
}
/**
 * This method was created by a SmartGuide.
 * @return int
 */
public int getMajorVersion() 
{
	return(majorVersion);
}
/**
 * This method returns the method entry at offset.
 */
public MethodEntry getMethod(int offset) 
{
	// the array containing the list of all the fields in the class.
	return(listOfMethods[offset]);
}
/**
 * This method returns the number of the methods in the class.
 * @return int
 */
public int getMethodCount() 
{
	return (listOfMethods.length);
}
/**
 * This method was created by a SmartGuide.
 * @return int
 */
public int getMinorVersion() 
{
	return(minorVersion);
}
/**
 * This method returns the classes modifiers as a string.
 * @return java.lang.String
 */
public String getModifiersAsString() 
{
	String returnValue = "";
	if((modifiers & ACC_PUBLIC) != 0)
		returnValue = "public ";
	if((modifiers & ACC_PRIVATE) != 0)
		returnValue = "private ";
	if((modifiers & ACC_PROTECTED) != 0)
		returnValue = "protected ";
	if((modifiers & ACC_STATIC) != 0)
		returnValue += "static ";
	if((modifiers & ACC_FINAL) != 0)
		returnValue += "final ";
	if((modifiers & ACC_VOLATILE) != 0)
		returnValue += "volatile ";
	if((modifiers & ACC_TRANSIENT) != 0)
		returnValue += "transient ";
	return (returnValue);
}
/**
 * This method returns the name of the super class.
 * @return java.lang.String
 */
public String getSuperClassName() 
{
	// the offset in the constant pool of the name of this class.
	return(constantPool[constantPool[superClassName].getInt()]).getString();
}
/**
 * This method loads a class file into memory and analizes it.
 * @param classFileName java.lang.String
 * @exception java.io.IOException The exception description.
 */
public void loadClass(String classFileName) 
	throws IOException 
{
	try
	{
		// if the file name is null throw an exception.
		if(classFileName == null) 
			throw (new IOException("Null submited as a file name"));

		// if the file does not exist then throw an exception.	
		File file = new File(classFileName);
		if(!file.isFile()) throw (new IOException("Can't find: " + classFileName));

		// build an input stream and buffer it.
		InputStream fileStream = new BufferedInputStream(new FileInputStream(file));
		
		// this stream reads the data from the array. 
		DataInputStream dataStream = new DataInputStream(fileStream);

		// read the header of the class file.
		// every class file must start with an integer whos hex value is 0xCAFEBABE.
		int CAFEBABE = dataStream.readInt();

		// if the signiture is not currect then throw an exception.
		if (CAFEBABE != JAVA_MAGIC) 
			throw new ClassFormatError("Not a class file.");

		// read the version number of the JVM used to build this class.
		minorVersion = dataStream.readShort();
		majorVersion = dataStream.readShort();

		// read the size of the constant pool
		int constantPoolSize = dataStream.readShort();

		constantPool = new ConstantPoolEntry[constantPoolSize];

		// iterateing through the constant pool starts at 1. Location 0 is reserved.
		for(int iter = 1; iter < constantPoolSize ; iter++)
			constantPool[iter] = ConstantPoolEntry.getConstantPoolEntry(dataStream.readByte(), dataStream);

		// the class's access flags.
		modifiers = dataStream.readShort();

		// the offset in the constant pool of the name of this class and its super class.
		className = dataStream.readShort();
		superClassName = dataStream.readShort();

		// pointers to the names of all the supported interfaces in the constantPool.
		supportedInterfaces = new int[dataStream.readShort()];
		
		// iterate through all the interfaces and load them
		for(int interfaceIter = 0; interfaceIter < supportedInterfaces.length ; interfaceIter++)
			supportedInterfaces[interfaceIter] = dataStream.readShort();
		
		// the array containing the list of all the fields in the class.
		int fieldsLength = dataStream.readShort();
		listOfFields = new FieldEntry[fieldsLength];
		
		// iterate through the fields and load them all to memory. 
		for(int fieldIter = 0; fieldIter < fieldsLength ; fieldIter++)
			listOfFields[fieldIter] = new FieldEntry(dataStream, constantPool);
		
		// the array containing the list of all the methods in the class.
		int methodLength = dataStream.readShort();
		listOfMethods = new MethodEntry[methodLength];

		// iterate through the methods and load them all to memory. 
		for(int methodIter = 0; methodIter < methodLength ; methodIter++)
			listOfMethods[methodIter] = new MethodEntry(dataStream, constantPool);

		// the number of attributes for this class
//		int attributeNumber = dataStream.readShort();

//		if(attributeNumber != 0)
		// the attribute table for this class file.
		attributeTable = new AttributeEntry(dataStream, constantPool);

		// close the data stream.
		dataStream.close();		
		fileStream.close();
	}	
	catch(ClassFormatError cerr)
	{
		System.out.println("Class format error: " + cerr.getMessage());
	}	
}


/**
 * this method was created for testing the VMAnalizer.
**/
public static void main(String argv[])
{
	try
	{
		if(argv.length == 1)
		{
			VMAnalizer classFile = new VMAnalizer(argv[0]);
			classFile.saveClass("c:/work/newFileName.class");
		}	
		else
		{
			System.out.println("this class takes one parameter which is the full class name (with the .class)");
		}	
	}	
	catch(IOException err)
	{
		System.out.println("An IOException occured: " + err.getMessage());
	}	
}	
/**
 * This method was created by a SmartGuide.
 * @param classFileName java.lang.String
 * @exception java.io.IOException The exception description.
 */
public void saveClass(String classFileName ) 
	throws IOException 
{
	try
	{
		// if the file name is null throw an exception.
		if(classFileName == null) 
			throw (new IOException("Null submited as a file name"));

		File file = new File(classFileName);

		// build an buffered output stream.
		OutputStream fileStream = new BufferedOutputStream(new FileOutputStream(file));
		
		// this stream reads the data from the array. 
		DataOutputStream dataStream = new DataOutputStream(fileStream);

		// write the header of the class file.
		// every class file must start with an integer whos hex value is 0xCAFEBABE.
		dataStream.writeInt(JAVA_MAGIC);

		// write the version number of the JVM used to build this class.
		dataStream.writeShort(minorVersion);
		dataStream.writeShort(majorVersion);

		// write the size of the constant pool
		dataStream.writeShort(constantPool.length);

		// iterateing through the constant pool starts at 1. Location 0 is reserved.
		for(int iter = 1; iter < constantPool.length ; iter++)
			constantPool[iter].save(dataStream);

		// set the classes the class's access flags.
		dataStream.writeShort(modifiers);

		// write the offset in the constant pool of the name of this class and its super class.
		dataStream.writeShort(className);
		dataStream.writeShort(superClassName);

		// write the pointers to the names of all the supported interfaces in the constantPool.
		dataStream.writeShort(supportedInterfaces.length);
		
		// iterate through all the interfaces and write them to the file.
		for(int interfaceIter = 0; interfaceIter < supportedInterfaces.length ; interfaceIter++)
			dataStream.writeShort(supportedInterfaces[interfaceIter]);
		
		// write the array containing the list of all the fields in the class.
		dataStream.writeShort(listOfFields.length);
		
		// iterate through the fields and load them all to memory. 
		for(int fieldIter = 0; fieldIter < listOfFields.length ; fieldIter++)
			listOfFields[fieldIter].save(dataStream);
		
		// write the array containing the list of all the methods in the class.
		dataStream.writeShort(listOfMethods.length);

		// iterate through the methods and write them all to the file. 
		for(int methodIter = 0; methodIter < listOfMethods.length ; methodIter++)
			listOfMethods[methodIter].save(dataStream);

		// the number of attributes for this class
//		int attributeNumber = dataStream.readShort();

		// the attribute table for this class file.
		attributeTable.save(dataStream);
		
		// close the data stream.
		dataStream.close();		
		fileStream.close();
	}	
	catch(ClassFormatError cerr)
	{
		System.out.println("Class format error: " + cerr.getMessage());
	}	
}
}